part of tecfy_core_client_package;

class EntityListController extends BaseController {
  Entity entity;
  List<dynamic> data;
  List<dynamic> filterdData;
  List<EntityFilter> filters;
  List<Property> searchableProperties;
  int pageIndex = 0;
  int pageIndexMax = 0;
  bool hasNextPage = true;
  ScrollController scrollController;

  TextEditingController searchController;
  RefreshController refreshController =
      RefreshController(initialRefresh: false); // for pull to refrsh

  EntityListController(State<StatefulWidget> state, String entityName)
      : super(state) {
    scrollController = ScrollController();
    refreshController.footerMode.addListener(footerListner);
    searchController = TextEditingController();

    filters = List.empty(growable: true); // List<EntityFilter>();
    print('-------------- EntityListController => $entityName');
    entity = EntityService.getEntity(entityName);
    searchableProperties =
        entity.properties.where((e) => e.isSearchable).toList();

    loadData();
  }

  @override
  dispose() {
    try {
      refreshController?.dispose();
    } catch (err) {}
    try {
      searchController?.dispose();
    } catch (err) {}
    try {
      scrollController?.dispose();
    } catch (err) {}
    super.dispose();
  }

  String sortProp;
  bool sortAsc = true;

  addChildren(items, List<dynamic> rows) {
    for (var item in items) {
      rows.add(item);
      if (item['_expanded'] == true &&
          item['_children'] != null &&
          item['_children'].length > 0) {
        rows = addChildren(item['_children'], rows);
      }
    }
    return rows;
  }

  Future onOpenFilter() async {
    var filterResult = await AppNavigator.showAppDialog(
      context,
      (c) => EntityFilterPage(entity.properties, filters),
    );
    print('--------------------$filterResult');
    if (filterResult != null) {
      filterResult = (filterResult as List<EntityFilter>)
          .where((e) =>
              e.value != null && (e.propertyName != null || e.property != null))
          .toList();
      if (filterResult.length != 0) {
        filters = filterResult;
        loadData();
      }
    }
  }

  Future globalSearchText() async {
    if (searchController.text.isEmpty) {
      filterdData = data;
    } else if (ConnectionService.isOfflineMode) {
      List<EntityFilter> f = [];
      for (var prop in entity.properties.where((p) => p.isSearchable)) {
        f.add(EntityFilter(
            propertyName: prop.dbName,
            operatorId: prop.valueType == ValueTypes.Number ? 'E' : 'C',
            value: searchController.text));
      }
      var d = (await EntityService.getData(entity,
              filter: f, pageIndex: pageIndex, filterOperator: 'or'))
          .data;
      if (d != null)
        filterdData = d;
      else
        filterdData = [];
    } else {
      filterdData = data.where((d) {
        return searchableProperties
            .any((p) => d[p.dbName].toString().contains(searchController.text));
      }).toList();
    }

    if (entity.isTree) {
      filterdData = addChildren(filterdData, List.empty(growable: true));
    }
    setState();
  }

  sortBy(Property prop) {
    if (prop.dbName != sortProp) {
      sortProp = prop.dbName;
      sortAsc = true;
    } else
      sortAsc = !sortAsc;

    data.sort((a, b) => sortAsc
        ? a[sortProp].toString().compareTo(b[sortProp].toString())
        : b[sortProp].toString().compareTo(a[sortProp].toString()));
    globalSearchText();
    setState();
  }

  bool _hasPageing = false;
  bool get hasPageing {
    if ((data?.length ?? 0) >= EntityService.pageSize) _hasPageing = true;
    return _hasPageing;
  }

  footerListner() async {
    print('==========on load ====== ${refreshController.footerStatus}');
    if (refreshController.footerStatus != LoadStatus.loading) return;
    var success = await gotoMextPageIndex();
    if (success && hasNextPage) {
      refreshController.loadComplete();
    } else if (success && !hasNextPage) {
      //refreshController.loadComplete();
      refreshController.loadNoData();
    } else {
      refreshController.loadFailed();
    }
  }

  Future<bool> gotoMextPageIndex() {
    pageIndex++;
    pageIndexMax = max(pageIndexMax, pageIndex);
    return loadData(withLoading: false);
  }

  gotoPageIndex(int i) {
    pageIndex = i;
    setLoading();
    loadData();
  }

  var isLoading = false;
  Future<bool> loadData({bool withLoading = true}) async {
    if (isLoading) return true;
    isLoading = true;
    var result = await EntityService.getData(entity,
        filter: filters, pageIndex: pageIndex, filterParentId: entity.isTree);
    if (withLoading) setResume();
    if (!result.success || result.data == null) {
      //data = [];
      isLoading = false;
      if (withLoading) {
        setState();
      }
      if (ConnectionService.isOnlineMode) {
        showMessage('Error Loading From Server');
      } else {
        showMessage('Error Loading From Local Database');
      }
      globalSearchText();
      return false;
    } else {
      if (LayoutService.isMobile(context) && pageIndex > 0 && !entity.isLocal)
        data.addAll(result.data);
      else
        data = result.data;

      if (result.data.length < EntityService.pageSize) hasNextPage = false;
    }
    globalSearchText();
    // if(pageIndex == 0) refreshAnimation();
    isLoading = false;
    return true;
  }

  Future loadFromServer() async {
    await loadData();
    refreshController.refreshCompleted();
  }

  Future onRefresh() async {
    pageIndex = 0;
    pageIndexMax = 0;
    hasNextPage = true;
    setLoading();
    await loadData();
  }

  Future onAdd() async {
    var result = await AppNavigator.navigateTo(
        context, () => EntityFormPage(entity.dbName),
        animationType: LayoutService.isMobile(context)
            ? AnimationTypes.slideUp
            : AnimationTypes.opacity);
    if (result == true) {
      loadFromServer();
    }
  }

  Future onUpdate(d) async {
    var result = await AppNavigator.navigateTo(
        context, () => EntityFormPage(entity.dbName, d),
        animationType: LayoutService.isMobile(context)
            ? AnimationTypes.slideUp
            : AnimationTypes.opacity);
    if (result == true) {
      loadFromServer();
    }
  }

  Future onDelete(d) async {
    if (!entity.permission.canDelete) {
      return showMessage('لا يوجد صلاحيه للحذف');
    }
    if (await confirmMessage('هل انت متاكد من الحذف؟')) {
      var result = await EntityService().delete(entity, d['id']);
      if (result.success) {
        showMessage('تم الحذف بنجاح', type: MessageType.info);
        data.removeWhere((e) => e['id'] == d['id']);
        globalSearchText();
        //loadFromServer();
      } else {
        showMessage('خطأ اثناء الحذف',
            message: 'الرجاء التاكد من ان العنصر غير مرتبط بعناصر اخرى',
            type: MessageType.error);
      }
    }
  }

  expandItem(Map<String, dynamic> d) async {
    try {
      if (!entity.isTree) return true;
      if (d['_children'] != null) {
        d['_expanded'] = !d['_expanded'];
        globalSearchText();
        return;
      }

      if (isLoading) return true;
      isLoading = true;
      print('expand ${d['id']}');
      setLoading();
      var result = await EntityService.getData(entity,
          filter: filters,
          pageIndex: pageIndex,
          filterParentId: true,
          parentId: d['id'].toString());

      setResume();
      isLoading = false;
      if (!result.success) {
        //data = [];
        return;
      }
      for (var r in result.data) {
        if (d['_tree_level'] != null)
          r['_tree_level'] = d['_tree_level'] + 1;
        else
          r['_tree_level'] = 1;
      }
      d['_children'] = result.data;
      d['_expanded'] = true;
      // data.insertAll(
      //     data.indexWhere((i) => i['id'] == d['id']) + 1, result.data);
      globalSearchText();
    } catch (err) {
      print(err);
      setResume();
      isLoading = false;
    }
  }

  lowerFirst(String s) {
    return '${s[0].toLowerCase()}${s.substring(1)}';
  }

  String get getPhotoDataPropName =>
      '${entity.properties.firstWhere((d) => d.isImage).dbName}Id';
}
