part of tecfy_core_client_package;

class Entity {
  String nameAr;
  String nameEn;
  String dbName;
  List<Property> properties;
  Map<String, Property> propertiesMap;
  List<Property> gridProperties;
  List<Property> editProperties;
  List<EntityAction> actions;
  Permission permission;
  String get name => LocalConfigService.isArabic ? nameAr : nameEn;

  bool hasPhoto;
  String photoPropName;
  Property displayFieldAr;
  Property displayFieldEn;
  Property get displayField =>
      LocalConfigService.isArabic ? displayFieldAr : displayFieldEn;

  bool isLocal = false;
  EntityTypes entityType;

  Entity.fromMap(Map<String, dynamic> row) {
    isLocal = row['isLocal'] ?? false;
    nameAr = row['nameAr'];
    nameEn = row['nameEn'];
    dbName = lowerFirst(row['dbName']);

    if (row['entityType'] != null)
      entityType = EntityTypes.values[row['entityType']];
    else
      entityType = EntityTypes.Data;

    if (row['permission'] != null)
      permission = Permission.fromMap(row['permission']);
    else
      permission = Permission();

    if (row['properties'] != null) {
      properties = row['properties']
          .map<Property>((p) => Property.getProperty(p))
          .toList();

      hasPhoto = properties?.any((d) =>
              d.propertyType == PropertyTypes.File &&
              (d as PropertyFile).isImage == true) ??
          false;
      if (hasPhoto)
        photoPropName =
            '${properties.firstWhere((d) => d.propertyType == PropertyTypes.File && (d as PropertyFile).isImage == true).dbName}Id';
      if (properties != null) {
        gridProperties = properties
            ?.where((p) =>
                p.isGrid == true &&
                (p.language == Language.notSelected ||
                    p.language == LocalConfigService.language))
            ?.toList();

        editProperties = properties?.where((p) => p.isEdit == true)?.toList();

        if (gridProperties?.length == 0) gridProperties = properties;
        editProperties.sort((a, b) => a.editOrder - b.editOrder);
        gridProperties.sort((a, b) => a.gridOrder - b.gridOrder);
      }
    }
    if (properties != null && properties.length > 0) {
      var displayFs = properties.where((e) => e.isDisplayField).toList();
      if (displayFs.length == 0) {
        displayFieldEn = properties.first;
        displayFieldAr = properties.first;
      } else if (displayFs.length == 1) {
        displayFieldEn = displayFs.first;
        displayFieldAr = displayFs.first;
      } else {
        displayFieldAr = displayFs.any((e) => e.language == Language.arabic)
            ? displayFs.firstWhere((e) => e.language == Language.arabic)
            : displayFs.first;
        displayFieldEn = displayFs.any((e) => e.language == Language.english)
            ? displayFs.firstWhere((e) => e.language == Language.english)
            : displayFs.last;
      }
    }
    if (row['actions'] != null) {
      actions = row['actions']
          .map<EntityAction>((p) => EntityAction.fromMap(p))
          .toList();
    }

    if (entityType == EntityTypes.ViewOnly ||
        entityType == EntityTypes.Lookup) {
      permission.canAdd = false;
      permission.canUpdate = false;
      permission.canDelete = false;
    }
  }

  lowerFirst(String s) {
    if (s == null || s.isEmpty) return '';
    return '${s[0].toLowerCase()}${s.substring(1)}';
  }

  Future<bool> setDefaultValues(data) async {
    bool updated = false;
    for (Property pro in properties) {
      if (await pro.getDefaultValue(data)) updated = true;
    }
    return updated;
  }

  bool _isTree;
  bool get isTree {
    if (_isTree == null && properties != null)
      _isTree = properties.any((d) => d.dbName == 'parentId');
    return _isTree ?? false;
  }

  List<Property> _calcProps;
  List<Property> get calcProps {
    if (_calcProps == null)
      _calcProps = properties
          .where((p) =>
              p.propertyType == PropertyTypes.Number &&
              (p as PropertyNumber).calcFormula != null &&
              (p as PropertyNumber).calcFormula.isNotEmpty)
          .toList();
    return _calcProps;
  }
}

enum EntityTypes {
  Lookup,
  Data,
  MenuItem,
  ViewOnly,
  SubEntity,
  SystemGanarated
}
