part of tecfy_core_client_package;

class DatabaseService {
  static Database _db;
  static String pathString;
  static String _dbPath = 'main.db';
  static LocalStorage storage;
  static bool isInit = false;
  static bool isEntityTableCreated = false;
  static bool get isLocalDbEnabled => _db != null;
  static List<DbTransaction> trans;
  static Future init() async {
    try {
      if (storage == null) storage = new LocalStorage('localDB.json');
    } catch (err) {}
    isInit = true;
    if (_db != null && _db.isOpen) {
      return;
    }
    if (!ConfigService.isLogin && !TecfyBasicApp.isTest) return;
    try {
      if (PlatformService.isSupportLocalDBSupported) {
        _dbPath = '${LocalConfigService.user.id}.tfd';
        pathString =
            '${(await path.getApplicationDocumentsDirectory()).path}${Platform.isWindows ? '\\' : '/'}';
        print('Database Path $pathString   $_dbPath');
        //await deleteDatabase(_dbPath);
        if (TecfyBasicApp.isTest)
          _db = await databaseFactoryFfi.openDatabase(inMemoryDatabasePath);
        else
          _db = await openDatabase(
              Platform.isWindows ? '$pathString$_dbPath' : _dbPath,
              version: 2,
              onCreate: (db, v) => _createDatabase(db, 0, v),
              onUpgrade: _createDatabase,
              onDowngrade: _createDatabase);
        print('db created');
        await _createDatabaseEntityTable(_db);
        print('db has been inited successfully !!!!!!!!!!!!!');
      } else {
        pathString = "d:\\";
      }
    } catch (err) {
      // cannot get path means it's windows
      pathString = "d:\\";
      print('xxxxxxxxxxxxxx  error inital database xxxxxxxxxxxx');
      print(err);
    }
  }

  static Future<bool> exeBatch(Future<bool> Function(Batch) process,
      [bool continueOnError = false]) async {
    try {
      if (_db == null) return false;
      var batch = _db.batch();
      if (await process(batch)) {
        batch.commit(continueOnError: continueOnError);
      }
      return true;
    } catch (err) {
      return false;
    }
  }

  static Future<List<Map<String, dynamic>>> rawQuery(String sql,
      [List<dynamic> arguments]) {
    if (_db == null || !_db.isOpen) {
      return Future.value([]);
    }
    return _db.rawQuery(sql, arguments);
  }

  static Future<int> insert(String table, Map<String, dynamic> values,
      {String nullColumnHack, ConflictAlgorithm conflictAlgorithm}) async {
    if (_db == null || !_db.isOpen) {
      print(
          'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX   db is not oppened!');
      return 0;
    }
    await _db.insert(
        table, toDatabaseEntity(EntityService.getEntity(table), values),
        nullColumnHack: nullColumnHack, conflictAlgorithm: conflictAlgorithm);
    return 10;
  }

  static Future<int> delete(String table,
      {String where, List<dynamic> whereArgs}) {
    if (_db == null || !_db.isOpen) {
      return Future.value(0);
    }
    return _db.delete(table, where: where, whereArgs: whereArgs);
  }

  static Future<int> update(String table, Map<String, dynamic> values,
      {String where,
      List<dynamic> whereArgs,
      ConflictAlgorithm conflictAlgorithm,
      bool skipDataUpdate = false}) {
    if (_db == null || !_db.isOpen) {
      return Future.value(0);
    }
    return _db.update(
        table,
        skipDataUpdate
            ? values
            : toDatabaseEntity(EntityService.getEntity(table), values),
        where: where,
        whereArgs: whereArgs,
        conflictAlgorithm: conflictAlgorithm);
  }

  static Future<List<Map<String, dynamic>>> query(String table,
      {bool distinct,
      List<String> columns,
      String where,
      List<dynamic> whereArgs,
      String groupBy,
      String having,
      String orderBy,
      int limit,
      int offset}) {
    if (_db == null || !_db.isOpen) {
      return Future.value([]);
    }
    return _db.query(table,
        distinct: distinct,
        columns: columns,
        where: where,
        whereArgs: whereArgs,
        groupBy: groupBy,
        limit: limit,
        orderBy: orderBy,
        offset: offset);
  }

  static Future<void> _createDatabase(
      Database _db, int _fromVerison, int _toVersion) async {
    try {
      // var fmVersion =0;
      // var appVersion =0;

      //if (_fromVerison <= 1) {
      await _db.execute(
          'CREATE TABLE IF NOT EXISTS Message (id TEXT PRIMARY KEY, fromUserId TEXT,toUserId TEXT,body TEXT, fromName TEXT,toName TEXT, time INTEGER,imSender INTEGER)');
      await _db.execute(
          'CREATE TABLE IF NOT EXISTS Notification (id TEXT PRIMARY KEY, titleAr TEXT,titleEn TEXT,messageAr TEXT, messageEn TEXT, time INTEGER, seen INTEGER, type INTEGER, payload TEXT)');
      await _db.execute(
          'CREATE TABLE IF NOT EXISTS Entity (id TEXT PRIMARY KEY, entity TEXT,body TEXT, timeStamp INTEGER, recordNumber INTEGER,isPosted INTEGER,postMessage TEXT)');
      await _db.execute(
          'CREATE TABLE IF NOT EXISTS DeleteLog (id TEXT PRIMARY KEY, entity TEXT, timeStamp INTEGER,isPosted INTEGER,postMessage TEXT)');

      await _db.execute(
          'CREATE INDEX IF NOT EXISTS  INX_Entity_entity ON Entity(entity,timeStamp)');
      await _db.execute(
          'CREATE INDEX IF NOT EXISTS  INX_Entity_isPosted ON Entity(isPosted)');
      await _db.execute(
          'CREATE INDEX IF NOT EXISTS  INX_DeleteLog_isPosted ON DeleteLog(isPosted)');

      //}
      await TecfyCoreApp.updateSqlLiteDb(_db, _fromVerison, _toVersion);
    } catch (err) {
      print('XXXXXXXXXXXXXXXXXXXXXXX databse create builtin tables');
      print(err);
    }
  }

  static List<String> builtInFields = [
    'id',
    'isPosted',
    'postMessage',
    'timeStamp'
  ];

  static Future<void> _createDatabaseEntityTable(Database _db) async {
    try {
      if (EntityService.entities == null) {
        print('XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX no entiotoies');
        return;
      }
      for (var entity in EntityService.entities.values) {
        List<String> cols = [
          'id TEXT PRIMARY KEY',
          'isPosted INTEGER',
          'postMessage TEXT',
          'timeStamp INTEGER'
        ];

        for (var prop in entity.properties) {
          if (builtInFields.contains(prop.dbName)) {
            continue;
          }
          if (prop.propertyType == PropertyTypes.File) {
            cols.add('${prop.dbName}Id TEXT');
            continue;
          }
          var type = 'TEXT';
          switch (prop.valueType) {
            case ValueTypes.DateTime:
              type = 'INTEGER';
              break;
            case ValueTypes.Boolean:
              type = 'INTEGER';
              break;
            case ValueTypes.Number:
              type = 'INTEGER';
              break;
            // case ValueTypes.NumberArray:
            //   type = 'TEXT';
            //   break;
            default:
          }
          cols.add('${prop.dbName} $type');
        }
        await _db.execute(
            'CREATE TABLE IF NOT EXISTS ${entity.dbName} (${cols.join(', ')})');
      }
      print(
          '------------------- all entity tables has been created ${EntityService.entities.values.length}');
    } catch (err) {
      print('XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX _createDatabaseEntityTable');
      print(err);
    }
  }

  static Future<void> deleteDatabaseFile() async {
    try {
      _db?.close();
      await deleteDatabase(_dbPath);
    } catch (err) {
      print('xxxxxxxxxxxxxx  error deleting database xxxxxxxxxxxx');
      print(err);
    }
  }

  static Future<bool> removeRange(String entity, List<EntityFilter> filters,
      {bool withTrans = false, String divider = 'and'}) async {
    var w = DatabaseService.getWhere(filters);

    if (withTrans)
      trans.add(DbTransaction(QueryType.delete,
          table: entity,
          where: w['where'].join(' $divider '),
          whereArgs: w['whereArgs']));
    else
      await _db.delete(entity,
          where: w['where'].join(' and '), whereArgs: w['whereArgs']);
    return true;
  }

  static dynamic getWhere(List<EntityFilter> filters) {
    var where = [];
    var whereArgs = [];

    if (filters != null)
      for (var f in filters) {
        var op = '=';
        if (f.operatorId == 'GE')
          op = '>=';
        else if (f.operatorId == 'LE')
          op = '<=';
        else if (f.operatorId == 'G')
          op = '>';
        else if (f.operatorId == 'C')
          op = 'like';
        else if (f.operatorId == 'NC')
          op = 'not like';
        else if (f.operatorId == 'L') op = '<';
        where.add('${f.property?.dbName ?? f.propertyName} $op ?');
        if (op == 'like' || op == 'not like')
          whereArgs.add('%${f.value}%');
        else
          whereArgs.add(f.value);
      }
    return {'where': where, 'whereArgs': whereArgs};
  }

  static Future<bool> saveEntity(
      String entityName, List<Map<String, dynamic>> data,
      {bool isPosted = true,
      bool overwriteLocal = false,
      bool withTrans = false}) async {
    if (entityName.startsWith('_')) {
      var old = (await readCollection(entityName)).data ?? [];
      old.removeWhere((e) => data.any((d) => e['id'] == d['id']));
      old.addAll(data);
      return (await writeCollection(entityName, old)).success;
    }
    if (!ConnectionService.isOfflineMode) return false;
    // print(
    //     'Start ----------------------- entity save $entityName ${data.length}');

    var ids = data.map<String>((d) => d['id']).join("','");

    if (!withTrans) trans = [];

    trans.add(DbTransaction(QueryType.delete,
        table: entityName,
        where: "${overwriteLocal ? '' : 'isPosted = 1 and '} id in ('$ids')"));

    var result = await addEntityAll(entityName, data,
        isPosted: isPosted, withTrans: true);
    if (!result || withTrans) return result;

    return await execTrans();
  }

  static Future<bool> addEntityAll(String entityName, List<dynamic> data,
      {bool isPosted = true, bool withTrans = false}) async {
    if (!ConnectionService.isOfflineMode) return false;
    try {
      var entity = EntityService.getEntity(entityName);

      List<Map<String, dynamic>> queryList = [];
      for (var d in data) {
        var newData = toDatabaseEntity(entity, d);
        newData['isPosted'] = isPosted ? 1 : 0;
        var query =
            "INSERT INTO $entityName (${newData.keys.join(',')}) VALUES(${newData.values.map((dd) => '?').join(',')}) ON CONFLICT(id) DO UPDATE SET ${newData.keys.where((k) => k != 'id').map((k) => '$k=excluded.$k').join(', ')};";
        queryList.add({'query': query, 'args': newData.values.toList()});
      }
      if (withTrans) {
        for (var q in queryList) {
          trans.add(DbTransaction(QueryType.rawQuery,
              query: q['query'], whereArgs: q['args']));
        }
        return true;
      } else {
        var result = await DatabaseService.exeBatch((batch) async {
          try {
            for (var q in queryList) {
              batch.execute(q['query'], q['args']);
            }
            await batch.commit();
            return true;
          } catch (err) {
            print(err);
            return false;
          }
        });
        return result;
      }
      // print(
      //     '---------------- entity saved $entityName ${dataList.length} $result');
    } catch (err) {
      print('XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX addEntityAll');
      print(err);
      return false;
    }
  }

  static Future<bool> saveEntitySingle(
      String entityName, Map<String, dynamic> d,
      {bool isPosted = true, bool withTrans: false}) async {
    if (d == null) return false;
    if (entityName.startsWith('_')) {
      var old = (await readCollection(entityName)).data ?? [];
      old.removeWhere((e) => e['id'] == d['id']);
      old.add(d);
      return (await writeCollection(entityName, old)).success;
    }

    if (!ConnectionService.isOfflineMode) return false;
    // print('Start ----------------------- entity single save $entityName ');
    var entity = EntityService.getEntity(entityName);
    var newData = d; //toDatabaseEntity(entity, d);
    newData['isPosted'] = isPosted ? 1 : 0;
    var isExistResult = await DatabaseService.rawQuery(
        'select count(*) from $entityName where id = ?', [d['id']]);
    bool isNew = false;
    if (isExistResult == null ||
        isExistResult.length == 0 ||
        isExistResult.first.values.first == 0) {
      isNew = true;
    }
    List<DbTransaction> t = [];
    if (isNew)
      t.add(DbTransaction(QueryType.insert, table: entityName, data: newData));
    else
      t.add(DbTransaction(QueryType.update,
          table: entityName,
          data: newData,
          where: 'id = ?',
          whereArgs: [d['id']]));

    bool result = true;
    if (withTrans) {
      trans.addAll(t);
    } else {
      result = await execTrans(t);
    }

    return result;
  }

  static Future<int> getCounts(String entity,
      {String where = '', List<dynamic> whereArgs}) async {
    try {
      if (entity.startsWith('_')) {
        return (await readCollection('_ServerConnections')).data.length;
      }
      //var entityStructure = EntityService.getEntity(entity);
      var q =
          'select count(*) from $entity ${where.isNotEmpty && !where.startsWith('where') ? 'where' : ''} $where';

      var result = await DatabaseService.rawQuery(q, whereArgs);
      if (result.length == 0) return -1;
      return result.first.values.first;
    } catch (err) {
      print('XXXXXXXXXXXXX DatabaseService.getCounts');
      print(err);
      return null;
    }
  }

  static Future<List<Map<String, dynamic>>> getEntities(String entity,
      {String where = '',
      List<dynamic> whereArgs,
      String orderBy = 'timeStamp',
      int offset,
      int limit}) async {
    try {
      if (entity.startsWith('_')) {
        return (await readCollection('_ServerConnections')).data;
      }
      // print('----local query----');
      // print(where);
      // print(whereArgs.map((a) => getDatabaseValue(a)).toList());
      var entityStructure = EntityService.getEntity(entity);
      var items = (await DatabaseService.query(entity,
              where: where.isEmpty ? null : where,
              whereArgs: whereArgs.map((a) => getDatabaseValue(a)).toList(),
              orderBy: orderBy,
              offset: offset,
              limit: limit))
          .map<Map<String, dynamic>>(
              (d) => fromDatabaseEntity(entityStructure, d))
          .toList();
      print(
          '----------------------------- get $entity from local DB (${items.length})');

      return items;
    } catch (err) {
      print('XXXXXXXXXXXXX DatabaseService.getEntities');
      print(err);
      return null;
    }
  }

  static Future<dynamic> getEntityById(String entity, String id) async {
    if (!ConnectionService.isOfflineMode) return null;
    var items =
        (await DatabaseService.query(entity, where: 'id = ?', whereArgs: [id]))
            //.map((d) => jsonDecode(d['body']))
            .toList();

    return items.length > 0
        ? fromDatabaseEntity(EntityService.getEntity(entity), items[0])
        : null;
  }

  static Future<OperationResult> deleteEntityById(String entity, String id,
      {bool withTrans = false}) async {
    if (entity.startsWith('_')) {
      var old = (await readCollection(entity)).data;
      old.removeWhere((e) => e['id'] == id);
      return (await writeCollection(entity, old));
    }
    if (!ConnectionService.isOfflineMode) return OperationResult(success: true);
    List<DbTransaction> t = [];
    t.add(DbTransaction(QueryType.rawQuery,
        query: 'delete from DeleteLog where isPosted = 1'));
    t.add(DbTransaction(QueryType.delete,
        table: 'DeleteLog', where: 'id = ?', whereArgs: [id]));

    t.add(DbTransaction(QueryType.insert, table: 'DeleteLog', data: {
      'id': id,
      'timeStamp': DateTime.now().millisecondsSinceEpoch,
      'entity': entity,
      'postMessage': '',
      'isPosted': 0
    }));
    t.add(DbTransaction(QueryType.delete,
        table: entity, where: 'id = ?', whereArgs: [id]));
    bool result = true;
    if (withTrans) {
      trans.addAll(t);
    } else {
      result = await execTrans(t);
    }

    return OperationResult(success: result);
  }

  static Future<bool> execTrans([List<DbTransaction> _trans]) async {
    print('|||||||||||||||||||||||||||||| Save DB Trans |||||||||||');
    List<DbTransaction> transactions = _trans ?? trans;
    print(transactions?.length ?? -1);
    if (transactions == null || transactions.length == 0) return false;
    try {
      if (_db == null) await init();
      await _db.transaction((txn) async {
        for (DbTransaction t in transactions) {
          switch (t.type) {
            case QueryType.insert:
              if (t.table == 'DeleteLog')
                await txn.insert(t.table, t.data);
              else
                await txn.insert(t.table,
                    toDatabaseEntity(EntityService.getEntity(t.table), t.data));
              break;
            case QueryType.delete:
              await txn.delete(t.table, where: t.where, whereArgs: t.whereArgs);
              break;
            case QueryType.update:
              await txn.update(t.table,
                  toDatabaseEntity(EntityService.getEntity(t.table), t.data),
                  where: t.where, whereArgs: t.whereArgs);
              break;
            case QueryType.rawQuery:
              await txn.execute(t.query, t.whereArgs);
              break;
            default:
          }
        }
      });
      return true;
    } catch (err, trace) {
      print('XXXXXXXXXXXXXXXXXXXXXXXXXXX exec trans error XXXXXXXX');
      print(err);
      print(trace);
      return false;
    }
  }

  static Future<OperationResult> deleteEntityBulk(List<String> ids) async {
    if (!ConnectionService.isOfflineMode) return OperationResult(success: true);
    await DatabaseService.exeBatch((b) async {
      for (var id in ids) {
        b.delete('Entity', where: 'id = ?', whereArgs: [id]);
      }
      await b.commit();
      return true;
    });
    return OperationResult(success: true);
  }

  // static Future<OperationResult> insert(
  //     String table, Map<String, dynamic> data) async {
  //   if (db == null || !db.isOpen) return OperationResult(success: false);
  //   var res = await db.insert(table, data);
  //   return OperationResult(success: res > 0);
  // }

  static Future<OperationResult> writeCollection(
      String entity, List<dynamic> data) async {
    try {
      ConfigService.setValueString(entity, json.encode(data));
      // if (storage != null) {
      //   if (await storage.ready) {
      //     storage.setItem(entity, data);
      //     //storage.dispose();
      //   }
      // } else {
      //   var file = File("$pathString$entity.json");
      //   await file.writeAsString(json.encode(data), flush: true);
      // }
      return OperationResult(success: true);
    } catch (err) {
      print('XXXXXXXXXXXXXXXXX  write local collection ($entity) $err');
      return OperationResult(success: false, message: err.toString());
    }
  }

  static Future<OperationResult<List<Map<String, dynamic>>>> readCollection(
      String entity) async {
    try {
      while (!isInit) await Future.delayed(Duration(milliseconds: 100));

      // print('--------------- read local collection ($entity)------------------');
      List<Map<String, dynamic>> data;
      data = (json.decode(ConfigService.getValueString(entity)) as List)
          .map<Map<String, dynamic>>((e) => e as Map<String, dynamic>)
          .toList();

      // if (storage != null) {
      //   //storage = new LocalStorage('localDB.json');
      //   if (await storage.ready) {
      //     data = storage.getItem(entity);
      //     //storage.dispose();
      //   }
      // } else {
      //   var file = File("$pathString$entity.json");
      //   data = json.decode(await file.readAsString());
      // }
      if (data == null) return OperationResult(success: false);
      return OperationResult(success: true, data: data);
    } catch (err) {
      print(err);
      //print('XXXXXXXXXXXXXXXXX  read local collection ($entity) ("$pathString$entity.json") $err');
      return OperationResult(success: false, message: err.toString());
    }
  }

  static Map<String, dynamic> fromDatabaseEntity(
      Entity entity, Map<String, dynamic> databaseEntity) {
    var newData = Map<String, dynamic>();
    if (databaseEntity['id'] == null) {
      print(
          'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX id = null ------fromDatabaseEntity');
      databaseEntity['id'] = Uuid().v4().replaceAll('-', '');
    }
    newData['id'] = databaseEntity['id'];

    if (databaseEntity['timeStamp'] != null) {
      if (databaseEntity['timeStamp'] is DateTime)
        newData['timeStamp'] =
            DateTime.fromMicrosecondsSinceEpoch(databaseEntity['timeStamp']);
    } else {
      newData['timeStamp'] = DateTime.now();
    }

    for (var prop in entity.properties) {
      if (prop.propertyType == PropertyTypes.File) {
        newData['${prop.dbName}Id'] = databaseEntity['${prop.dbName}Id'];
        continue;
      }
      if (databaseEntity[prop.dbName] == null ||
          builtInFields.contains(prop.dbName)) {
        continue;
      }
      switch (prop.valueType) {
        case ValueTypes.Boolean:
          newData[prop.dbName] = (databaseEntity[prop.dbName] == 1);
          break;
        case ValueTypes.DateTime:
          newData[prop.dbName] =
              DateTime.fromMillisecondsSinceEpoch(databaseEntity[prop.dbName]);
          break;
        case ValueTypes.Object:
          if (databaseEntity[prop.dbName] != null)
            newData[prop.dbName] = json.decode(databaseEntity[prop.dbName]);
          break;
        case ValueTypes.ObjectArray:
          if (databaseEntity[prop.dbName] != null)
            newData[prop.dbName] = json.decode(databaseEntity[prop.dbName]);
          break;
        case ValueTypes.Number:
          newData[prop.dbName] = databaseEntity[prop.dbName];
          break;
        default:
          newData[prop.dbName] = databaseEntity[prop.dbName].toString();
      }
    }
    return newData;
  }

  static dynamic getDatabaseValue(dynamic val) {
    if (val is bool)
      return val == true ? 1 : 0;
    else if (val is DateTime)
      return (val as DateTime).millisecondsSinceEpoch;
    else if (val is Map<String, dynamic>)
      return json.encode(val);
    else
      return val.toString();
  }

  static Map<String, dynamic> toDatabaseEntityEx(
      Map<String, dynamic> entityData) {
    if (entityData == null) return null;

    if (entityData['id'] == null) {
      print(
          'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX id = null ------toDatabaseEntityEx');
      entityData['id'] = Uuid().v4().replaceAll('-', '');
    }

    if (entityData['timeStamp'] != null) {
      if (entityData['timeStamp'] is DateTime)
        entityData['timeStamp'] =
            (entityData['timeStamp'] as DateTime).millisecondsSinceEpoch;
      else {
        var date = BaseModel.parseUTC(entityData['timeStamp'].toString());
        if (date != null) entityData['timeStamp'] = date.millisecondsSinceEpoch;
      }
    } else {
      entityData['timeStamp'] = DateTime.now().millisecondsSinceEpoch;
    }

    for (var key in entityData.keys) {
      if (entityData[key] == null) continue;
      // if (builtInFields.contains(key)) {
      //   continue;
      // }
      if (entityData[key] is bool)
        entityData[key] = entityData[key] == true ? 1 : 0;
      else if (entityData[key] is DateTime)
        entityData[key] = (entityData[key] as DateTime).millisecondsSinceEpoch;
      else if (entityData[key] is Map<String, dynamic>)
        entityData[key] = json.encode(entityData[key]);
    }
    return entityData;
  }

  static Map<String, dynamic> toDatabaseEntity(
      Entity entity, Map<String, dynamic> entityData) {
    if (entity == null || entityData == null) return null;
    var newData = Map<String, dynamic>();
    if (entityData['id'] == null) {
      print(
          'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX id = null ------toDatabaseEntity');
      entityData['id'] = Uuid().v4().replaceAll('-', '');
    }
    newData['id'] = entityData['id'];
    if (entityData['isPosted'] == null)
      newData['isPosted'] = 0;
    else if (entityData['isPosted'] is bool)
      newData['isPosted'] = (entityData['isPosted'] ?? false) ? 1 : 0;
    else if (entityData['isPosted'] is num)
      newData['isPosted'] = entityData['isPosted'] ?? 0;

    newData['postMessage'] = entityData['postMessage'];

    if (entityData['timeStamp'] != null) {
      if (entityData['timeStamp'] is DateTime)
        newData['timeStamp'] =
            (entityData['timeStamp'] as DateTime).millisecondsSinceEpoch;
      else {
        var date = BaseModel.parseUTC(entityData['timeStamp'].toString());
        if (date != null) newData['timeStamp'] = date.millisecondsSinceEpoch;
      }
    } else {
      newData['timeStamp'] = DateTime.now().millisecondsSinceEpoch;
    }

    for (var prop in entity.properties) {
      if (prop.propertyType == PropertyTypes.File) {
        newData['${prop.dbName}Id'] = entityData['${prop.dbName}Id'];
        continue;
      }
      if (builtInFields.contains(prop.dbName)) {
        continue;
      }
      switch (prop.valueType) {
        case ValueTypes.Boolean:
          newData[prop.dbName] = entityData[prop.dbName] == true ? 1 : 0;
          break;
        case ValueTypes.DateTime:
          if (entityData[prop.dbName] is DateTime)
            newData[prop.dbName] =
                (entityData[prop.dbName] as DateTime).millisecondsSinceEpoch;
          else {
            var date = BaseModel.parseUTC(entityData[prop.dbName].toString());
            if (date != null)
              newData[prop.dbName] = date.millisecondsSinceEpoch;
          }
          break;
        case ValueTypes.Object:
          if (entityData[prop.dbName] != null) {
            newData[prop.dbName] = json.encode(entityData[prop.dbName]);
          }
          break;
        case ValueTypes.ObjectArray:
          if (entityData[prop.dbName] != null) {
            newData[prop.dbName] = json.encode(entityData[prop.dbName]);
          }
          break;
        case ValueTypes.Number:
          newData[prop.dbName] = entityData[prop.dbName];
          break;
        default:
          if (entityData[prop.dbName] != null)
            newData[prop.dbName] = entityData[prop.dbName].toString();
      }
    }
    return newData;
  }
}

class DbTransaction {
  String query;
  QueryType type;
  String table;
  String where;
  List<dynamic> whereArgs;
  Map<String, dynamic> data;
  DbTransaction(this.type,
      {this.query, this.where, this.whereArgs, this.table, this.data});
}

enum QueryType { insert, update, delete, rawQuery }
