part of tecfy_core_client_package;

class EntityService {
  static List<EntityCategory> categories;
  //static List<Entity> entities;
  static Map<String, Entity> entities;
  static Map<String, List<dynamic>> dataCache = {};
  static int pageSize = 15;

  static dynamic getValue(Map<String, dynamic> data, value) {
    if (data.keys.contains(value)) return data[value];
    if (value == 'true') return true;
    if (value == 'false') return true;
    if (num.tryParse(value) != null) return num.tryParse(value);
    return null;
  }

  static Future initEntities() async {
    try {
      var result =
          await TecfyBasicApp.apiService.httpGetDynamic('entity/GetEntities');
      if (result.success) {
        ConfigService.setValueString('Entity', jsonEncode(result.data));
      }
      if (!result.success) {
        result.data = jsonDecode(ConfigService.getValueString('Entity'));
        result.success = (result.data != null);
      }
      if (result.success) {
        print('-----------  GetEntities ------------ ${result.data.length}');

        List<Entity> entList = [];
        EntityService.entities = {};
        for (var cat in result.data) {
          var ent = Entity.fromMap(cat);
          entList.add(ent);
          EntityService.entities[ent.dbName] = ent;
        }

        print(EntityService.entities.keys.length);
        for (var c in EntityService.categories) {
          c.entities = entList
              .where((e) => c.entities.any((ee) => ee.dbName == e.dbName))
              .toList();
        }
        print('categories is ${EntityService.categories.length}');
      }
    } catch (err) {
      print('XXXXXXXXXXXXXXX GetEntities $err');
      throw "init entities Error: $err";
    }
  }

  static Future<OperationResult<List<dynamic>>> getDataEx(String entity,
      {List<EntityFilter> filter,
      int pageIndex = 0,
      bool activeOnly = false,
      bool allRecords = false,
      String parentId,
      bool filterParentId = false}) {
    return getData(EntityService.getEntity(entity),
        filter: filter,
        pageIndex: pageIndex,
        activeOnly: activeOnly,
        allRecords: allRecords,
        parentId: parentId,
        filterParentId: filterParentId);
  }

  static Future<OperationResult<List<Map<String, dynamic>>>> getData(
      Entity entity,
      {List<EntityFilter> filter,
      int pageIndex = 0,
      bool activeOnly = false,
      bool allRecords = false,
      String parentId,
      bool filterParentId = false,
      String filterOperator = 'and'}) async {
    try {
      if (entity == null) {
        return OperationResult(
            success: false, message: 'GetData: Entity cannot be null');
      }
      print('start get data');
      if (ConnectionService.isOfflineMode || entity.isLocal) {
        var where = [];
        var whereArgs = [];
        if (filterParentId && parentId != null && parentId.isNotEmpty) {
          where.add('parentId = ?');
          whereArgs.add(parentId);
        }
        var w = DatabaseService.getWhere(filter);
        where.addAll(w['where']);
        whereArgs.addAll(w['whereArgs']);
        var dbResult = await DatabaseService.getEntities(entity.dbName,
            where: where.join(' $filterOperator '),
            whereArgs: whereArgs,
            limit: allRecords ? 10000 : pageSize,
            offset: pageIndex * pageSize);
        if (dbResult != null) {
          dbResult.sort((a, b) => (a['sortId'] ?? 1) - (b['sortId'] ?? 1));
          dbResult = await bindOtherEntities(entity, dbResult);
          return OperationResult<List<Map<String, dynamic>>>(
              success: true, data: dbResult);
        }
        if (entity.isLocal)
          return OperationResult<List<Map<String, dynamic>>>(
              success: false, message: 'No Records in Local Database!');
      }
      var pagingQuery =
          '?skip=${pageIndex * pageSize}&count=${allRecords ? 10000 : pageSize}&activeOnly=$activeOnly&filterParentId=$filterParentId';
      if (parentId != null) pagingQuery += '&parentId=$parentId';

      var result = await TecfyBasicApp.apiService.httpGetDynamic(
          'entity/get/${entity.dbName}' +
              ((filter?.length ?? 0) > 0
                  ? '$pagingQuery&filter=${jsonEncode(filter.map<Map>((f) => f.toJson()).toList())}'
                  : pagingQuery));
      if (!result.success) {
        print(result.success);
        print(result.message);
        return OperationResult<List<Map<String, dynamic>>>(
            success: false, message: result.message);
      }
      // if(cache) dataCache[entity] =result.data;
      // if (pageIndex == 0)

      //   await DatabaseService.writeCollection(entity.dbNme.saveEntity(entity.dbName, result.data,
      //     overwriteLocal: entity.isLocal);
      return OperationResult<List<Map<String, dynamic>>>(
          success: true, data: result.data.cast<Map<String, dynamic>>());
    } catch (err) {
      print('XXXXXXXXXXXXXXXXXXXXX EntotyService.getData');
      print(err);
      return OperationResult<List<Map<String, dynamic>>>(
          success: false, message: err.toString());
    }
  }

  static Future<List<Map<String, dynamic>>> bindOtherEntities(
      Entity entity, List<Map<String, dynamic>> _data) async {
    if (ConnectionService.isOnlineMode) return _data;
    for (var prop in entity.properties) {
      if (!prop.isEdit && !prop.isGrid) continue;
      if (prop.propertyType == PropertyTypes.Dropdown) {
        Map<String, dynamic> cache = {};
        var property = (prop as PropertyDropdown);
        for (var r in _data) {
          if (r[property.dbNameEntity] != null) continue;
          if (cache[r[prop.dbName]] == null) {
            cache[r[prop.dbName]] = (await EntityService.getOne(
                property.entityName, r[prop.dbName]));
          }
          r[property.dbNameEntity] = cache[r[prop.dbName]];
        }
      }
    }
    return _data;
  }

  static Future<Map<String, dynamic>> getOne(String entityName, dynamic id,
      {bool forView = false, bool withRelatedData = false}) async {
    if (ConnectionService.isOfflineMode) {
      var dbResult = await DatabaseService.getEntityById(entityName, id);
      if (dbResult == null || !withRelatedData) return dbResult;
      var entity = getEntity(entityName);
      for (var prop in entity.properties) {
        if (prop.propertyType == PropertyTypes.OtherEntityMany) {
          dbResult[prop.dbName] = (await getDataEx(
                  (prop as PropertyOtherEntityMany).entityName,
                  filter: [
                EntityFilter(
                    value: id,
                    operatorId: "E",
                    propertyName: "${entity.dbName}Id")
              ]))
              .data;
        }
      }
      return dbResult;
    } else {
      var result = await TecfyBasicApp.apiService
          .httpGetDynamic('entity/GetById/$entityName/$id?view=$forView');
      return result.data;
    }
  }

  Future<OperationResult> delete(Entity entity, dynamic id,
      {bool withTrans = false}) async {
    IEntityTransaction service =
        TecfyCoreApp.entityServices[entity.dbName.toLowerCase()];
    OperationResult result;
    if (!withTrans) DatabaseService.trans = [];
    if (service != null) {
      var serviceResult = await service.beforeDelete(null, id);
      if (!serviceResult.success) {
        return serviceResult;
      }
    }

    if (ConnectionService.isOfflineMode || entity.isLocal) {
      result = await DatabaseService.deleteEntityById(entity.dbName, id,
          withTrans: true);
    } else {
      result = await TecfyBasicApp.apiService
          .httpDeleteDynamic('entity/${entity.dbName}/$id');
    }
    if (!result.success) return result;
    if (service != null) {
      var serviceResult = await service.afterDelete(null, id);
      if (!serviceResult.success) {
        return serviceResult;
      }
    }

    return OperationResult(
        success: withTrans || ConnectionService.isOnlineMode
            ? true
            : await DatabaseService.execTrans());
  }

  static Entity getEntity(String name) {
    if (name == '_TestEntity') {
      return Entity.fromMap({
        'dbName': '_TestEntity',
        'nameAr': 'TestEntity',
        'nameEn': 'TestEntity',
        'isLocal': true,
        'permission': {
          'canView': true,
          'canUpdate': true,
          'canAdd': true,
          'canDelete': true
        },
        'properties': [
          {
            'dbName': 'name',
            'nameAr': 'Name',
            'nameEn': 'Name',
            'valueType': ValueTypes.Text.index,
            'propertyType': PropertyTypes.Text.index,
            'fullWidth': false,
            'isEdit': true,
            'isGrid': true,
            'isRequired': true,
            'isSearchable': true,
            'isDisplayField': true
          },
          {
            'dbName': 'description',
            'nameAr': 'description',
            'nameEn': 'description',
            'valueType': ValueTypes.Text.index,
            'propertyType': PropertyTypes.Text.index,
            'fullWidth': false,
            'isEdit': true,
            'isGrid': true,
            'isRequired': true,
            'isSearchable': true,
            'isDisplayField': true
          },
          {
            'dbName': 'area',
            'nameAr': 'Area',
            'nameEn': 'Area',
            'valueType': ValueTypes.Text.index,
            'propertyType': PropertyTypes.MarkImage.index,
            'fullWidth': true,
            'isEdit': true,
            'isGrid': false,
            'isRequired': true,
            'isSearchable': false,
            'isDisplayField': false
          }
        ]
      });
    }
    if (name == '_ServerConnections') {
      return Entity.fromMap({
        'isLocal': true,
        'nameAr': 'قنوات الاتصال',
        'nameEn': 'Server Connections',
        'dbName': '_ServerConnections',
        'permission': {
          'canView': true,
          'canUpdate': true,
          'canAdd': true,
          'canDelete': true
        },
        'properties': [
          {
            'dbName': 'name',
            'nameAr': 'الاسم',
            'nameEn': 'Name',
            'valueType': ValueTypes.Text.index,
            'propertyType': PropertyTypes.Text.index,
            'fullWidth': false,
            'isEdit': true,
            'isGrid': true,
            'isRequired': true,
            'isSearchable': true,
            'isDisplayField': true
          },
          {
            'dbName': 'server',
            'nameAr': 'عنوان الخادم',
            'nameEn': 'Server',
            'valueType': ValueTypes.Text.index,
            'propertyType': PropertyTypes.Text.index,
            'fullWidth': false,
            'isEdit': true,
            'isGrid': false,
            'isRequired': true,
            'isSearchable': false,
            'isDisplayField': false
          },
          {
            'dbName': 'sortId',
            'nameAr': 'الترتيب',
            'nameEn': 'Sort Id',
            'valueType': ValueTypes.Number.index,
            'propertyType': PropertyTypes.Number.index,
            'fullWidth': false,
            'isEdit': true,
            'isGrid': false,
            'isRequired': true,
            'isSearchable': false,
            'isDisplayField': false
          },
          {
            'dbName': 'version',
            'nameAr': 'الاصدار',
            'nameEn': 'Version',
            'valueType': ValueTypes.Text.index,
            'propertyType': PropertyTypes.Text.index,
            'fullWidth': false,
            'isEdit': false,
            'isGrid': true,
            'isRequired': false,
            'isSearchable': false,
            'isDisplayField': false
          },
          {
            'dbName': 'lastActive',
            'nameAr': 'اخر وقت للفحص',
            'nameEn': 'Last Active',
            'valueType': ValueTypes.Number.index,
            'propertyType': PropertyTypes.DateTime.index,
            'fullWidth': false,
            'isEdit': false,
            'isGrid': true,
            'isRequired': false,
            'isSearchable': false,
            'isDisplayField': false
          },
          {
            'dbName': 'isCloud',
            'nameAr': 'هل سحابي؟',
            'nameEn': 'Is Cloud',
            'valueType': ValueTypes.Boolean.index,
            'propertyType': PropertyTypes.Boolean.index,
            'fullWidth': false,
            'isEdit': true,
            'isGrid': false,
            'isRequired': true,
            'isSearchable': false,
            'isDisplayField': false
          },
          {
            'dbName': 'isHTTPS',
            'nameAr': 'HTTPS',
            'nameEn': 'HTTPS',
            'valueType': ValueTypes.Boolean.index,
            'propertyType': PropertyTypes.Boolean.index,
            'fullWidth': false,
            'isEdit': true,
            'isGrid': false,
            'isRequired': true,
            'isSearchable': false,
            'isDisplayField': false
          },
          {
            'dbName': 'active',
            'nameAr': 'فعال',
            'nameEn': 'Active',
            'valueType': ValueTypes.Boolean.index,
            'propertyType': PropertyTypes.Boolean.index,
            'fullWidth': false,
            'isEdit': true,
            'isGrid': false,
            'isRequired': true,
            'isSearchable': false,
            'isDisplayField': false
          },
          {
            'dbName': 'colorString',
            'nameAr': 'اللون',
            'nameEn': 'Color',
            'valueType': ValueTypes.Text.index,
            'propertyType': PropertyTypes.Color.index,
            'fullWidth': false,
            'isEdit': true,
            'isGrid': false,
            'isRequired': false,
            'isSearchable': false,
            'isDisplayField': false
          },
        ],
      });
    }
    return entities[lowerFirst(name)];
  }

  static OperationResult validateEntity(Entity entity, dynamic data) {
    print('Validate Entity : ${entity.dbName}');
    try {
      for (var prop in entity.properties
          .where((e) => e.isEdit == true && e.readOnly == false)) {
        if (prop.propertyType == PropertyTypes.OtherEntityMany) {
          var subList = data[prop.dbName];
          if (prop.isRequired && (subList == null || subList.length == 0))
            return OperationResult(
                success: false,
                message: 'Add Atleast one record: ${prop.nameEn}');
          else if (subList != null)
            for (var sub in subList) {
              var result = validateEntity(
                  getEntity((prop as PropertyOtherEntityMany).entityName), sub);
              if (!result.success) return result;
            }
        } else {
          var result = prop.isValid(data[prop.dbName]);
          if (!result.success) {
            print('------Error validating: ${prop.dbName}');
            print(data[prop.dbName]);
            print(result.message);
            return result;
          }
        }
      }
      return OperationResult(success: true);
    } catch (err) {
      return OperationResult(
          success: false,
          message: 'Error in Validation ${entity.dbName}: ${err.toString()}');
    }
  }

  static Future<OperationResult> _addEntity(Entity entity, data,
      {bool localOnly = false,
      bool withTrans = false,
      bool isPosted = false}) async {
    print('+++++++++++++++++++++ Start Add ${entity.dbName}');
    if (entity.dbName.startsWith('_')) {
      var localItems = await DatabaseService.readCollection(entity.dbName);
      if (!localItems.success) localItems.data = [];
      if (data['id'] == null) data['id'] = Uuid().v4().replaceAll('-', '');
      localItems.data.add(data);
      var saveResult =
          await DatabaseService.writeCollection(entity.dbName, localItems.data);
      if (saveResult.success) ConnectionService.reloadConnections();
      return saveResult;
    }
    if (ConnectionService.isOfflineMode || entity.isLocal) {
      if (isPosted)
        data['isPosted'] = true;
      else
        data['isPosted'] = data['isPosted'] ?? false;
      if (withTrans)
        DatabaseService.trans.add(
            DbTransaction(QueryType.insert, table: entity.dbName, data: data));
      else {
        var r = await DatabaseService.insert(entity.dbName, data);
        return OperationResult(success: r > 0);
      }
      return OperationResult(success: true);
    } else {
      var result = await TecfyBasicApp.apiService.httpPostEx(
          'entity/${entity.dbName}',
          json.encode(data, toEncodable: customEncode).replaceAll('\"', '\''));
      if (!result.success)
        print(result.toString());
      else
        print('${entity.dbName} added successfully');
      return result;
    }
  }

  static Future<OperationResult> _updateEntity(Entity entity, data,
      {bool localOnly = false,
      bool withTrans = false,
      bool isPosted = false}) async {
    print('+++++++++++++++++++++ Start Update ${entity.dbName}');
    if (entity.dbName.startsWith('_')) {
      var localItems = await DatabaseService.readCollection(entity.dbName);
      if (!localItems.success) return localItems;
      if (data['id'] == null) data['id'] = Uuid().v4().replaceAll('-', '');
      localItems.data.removeWhere((element) => element['id'] == data['id']);
      localItems.data.add(data);
      localItems.data.sort((a, b) => a['sortId'] ?? 0 - b['sortId'] ?? 0);
      var saveResult =
          await DatabaseService.writeCollection(entity.dbName, localItems.data);
      if (saveResult.success) ConnectionService.reloadConnections();
      return saveResult;
    }

    if (ConnectionService.isOfflineMode || entity.isLocal) {
      if (withTrans)
        DatabaseService.trans.add(DbTransaction(QueryType.update,
            table: entity.dbName,
            data: data,
            where: 'id = ?',
            whereArgs: [data['id']]));
      else {
        var r = await DatabaseService.update(entity.dbName, data,
            where: 'id = ?', whereArgs: [data['id']]);
        return OperationResult(success: r > 0);
      }
      return OperationResult(success: true);
    } else {
      var result = await TecfyBasicApp.apiService.httpPutEx(
          'entity/${entity.dbName}',
          json.encode(data, toEncodable: customEncode).replaceAll('\"', '\''));
      return result;
    }
  }

  static Future<OperationResult> saveEntityEx(
      String entity, dynamic data, BuildContext context, bool isAddNew,
      {bool skipValidation = false,
      bool withTrans = false,
      bool isPosted = false}) {
    return EntityService.saveEntity(
        EntityService.getEntity(entity), data, context, isAddNew,
        withTrans: withTrans, isPosted: isPosted);
  }

  static Future<OperationResult> saveEntity(
      Entity entity, dynamic data, BuildContext context, bool isAddNew,
      {bool skipValidation = false,
      bool localOnly = false,
      bool withTrans = false,
      bool isPosted = false}) async {
    try {
      if (data['isLocked'] == true && !isAddNew) {
        return OperationResult(success: false, message: 'Record is Locked');
      }
      if (!withTrans) DatabaseService.trans = [];
      print(
          '+++++++++++++++++++++SAVE Start [${isAddNew ? 'Add' : 'Update'}] [${entity.dbName}]');
      if (!skipValidation) {
        var validationResult = EntityService.validateEntity(entity, data);
        if (!validationResult.success) return validationResult;
      }
      IEntityTransaction service =
          TecfyCoreApp.entityServices[entity.dbName.toLowerCase()];
      OperationResult result;
      print('--------- Service ${service == null ? '[NOT]' : ''} Found');
      if (isAddNew) {
        if (service != null) {
          var serviceResult = await service.beforeAdd(context, data);
          if (!serviceResult.success) {
            return serviceResult;
          }
          if (!serviceResult.skip && serviceResult.success) {
            data = serviceResult.data;
            // result = await EntityService._addEntity(entity, serviceResult.data,
            //     localOnly: localOnly, withTrans: true);
          }
        }
        if (result == null) {
          result = await EntityService._addEntity(entity, data,
              localOnly: localOnly, withTrans: true, isPosted: isPosted);
        }
      } else {
        if (service != null) {
          var serviceResult = await service.beforeUpdate(context, data);

          if (!serviceResult.success) {
            return serviceResult;
          }
          if (!serviceResult.skip && serviceResult.success) {
            data = serviceResult.data;
            // result = await EntityService._updateEntity(
            //     entity, serviceResult.data,
            //     localOnly: localOnly, withTrans: true);
          }
        }
        result = await EntityService._updateEntity(entity, data,
            localOnly: localOnly, withTrans: true, isPosted: isPosted);
      }

      if (!result.success) return result;

      if (ConnectionService.isOfflineMode)
        for (var prop in entity.properties) {
          if (data[prop.dbName] == null) continue;
          if (prop.isEdit && prop.propertyType == PropertyTypes.ManyToMany) {
            var property = prop as PropertyManyToMany;
            //var dataList = data
            if (property.useArrayStringValue) {
              var newVal = data[prop.dbName] is List<String>
                  ? data[prop.dbName] as List<String>
                  : (data[prop.dbName] as List<dynamic>).cast<String>();
              data[prop.dbName + "Value"] = newVal.join(',');
              continue;
            }
            var manyToManyUpdateResult =
                await EntityService._updateManyToManyEntity(
                    prop, data, data['id']);
            if (!manyToManyUpdateResult.success) {
              return manyToManyUpdateResult;
            }
          } else if (prop.isEdit &&
              prop.propertyType == PropertyTypes.OtherEntityMany) {
            var rows = data[prop.dbName];
            if (rows == null) continue;
            var property = prop as PropertyOtherEntityMany;

            var otherEntity = EntityService.getEntity(property.entityName);
            // read the old rows

            var otherPropertyType = "${entity.dbName}Id";

            List<Map<String, dynamic>> oldRows = isAddNew
                ? []
                : (await getData(otherEntity, filter: [
                    EntityFilter(
                        value: data['id'],
                        operatorId: "E",
                        propertyName: "${entity.dbName}Id")
                  ]))
                    .data;

            // update the updated
            for (var row in rows) {
              // set the relation key
              row[otherPropertyType] = data['id'];
              bool isExist = oldRows == null
                  ? false
                  : oldRows.any((e) => e['id'] == row['id']);
              // add or update
              OperationResult updateResult = await EntityService.saveEntity(
                  otherEntity, row, null, !isExist,
                  withTrans: true);
              if (!updateResult.success) {
                return updateResult;
              }
            }

            if (oldRows != null && oldRows.length > 0) {
              // delete the removed
              var removedRows = oldRows
                  .where((r) => !rows.any((e) => e['id'] == r['id']))
                  .map((r) => EntityFilter(
                      value: r['id'], operatorId: 'E', propertyName: 'id'))
                  .toList();

              if (removedRows.length > 0) {
                var removeResult = await DatabaseService.removeRange(
                    otherEntity.dbName, removedRows,
                    withTrans: true, divider: 'or');

                if (!removeResult)
                  return OperationResult(
                      success: false,
                      message: 'unable to delete other entity ${prop.dbName}');
              }
            }
            // remove it from the model because it's already saved
            data[prop.dbName] = null;
          }
        }

      // after add or update
      if (service != null) {
        OperationResultEntityTransaction serviceResult;
        if (isAddNew)
          serviceResult = await service.afterAdd(context, data);
        else
          serviceResult = await service.afterUpdate(context, data);

        if (!serviceResult.success) {
          return serviceResult;
        }
      }
      return OperationResult(
          success: withTrans || ConnectionService.isOnlineMode
              ? true
              : await DatabaseService.execTrans(),
          data: result.data);
    } catch (err, tack) {
      print('XXXXXXXXXXXXXXX Error Save Entity: $err');
      print(tack);
      print(data);
      return OperationResult(success: false, message: 'Error: $err');
    }
  }

  static dynamic customEncode(dynamic item) {
    if (item is DateTime) {
      return intl.DateFormat("yyyy-MM-ddTHH:mm:ss").format(item);
    }
    return item;
  }

  static String lowerFirst(String s) {
    if (s == null || s.isEmpty) return '';
    return '${s[0].toLowerCase()}${s.substring(1)}';
  }

  static upperFirst(String s) {
    if (s == null || s.isEmpty) return '';
    return '${s[0].toUpperCase()}${s.substring(1)}';
  }

  static bool getBool(dynamic data) {
    if (data == null) return false;
    if (data is bool) return (data == true);
    if (data is num) return (data == 1);
    return data.toString() == '1' || data.toString() == 'true';
  }

  static Future<OperationResult> _updateManyToManyEntity(
      PropertyManyToMany prop, Map<String, dynamic> model, String parentId,
      {bool withTrans = false}) async {
    print('--------------- start save many to many ${prop.dbName}');
    try {
      var middleEntityType = EntityService.getEntity(prop.middleEntityName);
      var oldItems = await getData(middleEntityType, filter: [
        EntityFilter(
            value: parentId,
            operatorId: "E",
            propertyName: prop.middlePropertyName)
      ]);

      var middleOtherPropertyNameProp = prop.middleOtherPropertyName;
      var oldIds =
          oldItems.data.map((s) => s[middleOtherPropertyNameProp]).toList();

      var propModel = prop.dbName;

      if (model[propModel] == null) {
        return OperationResult(success: true);
      }

      var newItems = model[propModel] is List<Map<String, dynamic>>
          ? model[propModel] as List<Map<String, dynamic>>
          : (model[propModel] as List<dynamic>).cast<Map<String, dynamic>>();
      // ((IEnumerable)propModel.GetValue(model)).OfType<object>();

      model[propModel] = null;

      //Type newItemType = null;
      var newItemIdProp;
      if (newItems.length > 0) {
        //newItemType = newItems.First().GetType();
        newItemIdProp = prop.middleOtherPropertyName; //Updated "Id"
      }
      List<dynamic> ids;
      if (prop.isWithData) {
        ids = newItems.map((s) => s[newItemIdProp]).toList();
      } else {
        ids = newItems.toList();
      }

      Entity middlePropertyStructure;
      if (prop.isWithData)
        middlePropertyStructure =
            EntityService.getEntity(prop.middleEntityName);

      // add the new selected items
      var newIds = ids.where((s) => !oldIds.contains(s) && s != null);
      if (newIds.length > 0) {
        for (var id in newIds) {
          dynamic newItem;
          if (prop.isWithData) {
            newItem = newItems.firstWhere((i) => i[newItemIdProp] == id);
          }

          dynamic middleEntity = {};

          var entity = middleEntityType;

          if (prop.isWithData) {
            for (var tempPropStructure in entity.properties.where((p) =>
                p.isEdit && p.propertyType != PropertyTypes.ManyToMany)) {
              var newValue = newItem[tempPropStructure.dbName];
              middleEntity[tempPropStructure.dbName] = newValue;
            }
          }

          middleEntity[prop.middlePropertyName] = parentId;
          middleEntity[prop.middleOtherPropertyName] = id;

          var newObject = await EntityService.saveEntity(
              middleEntityType, middleEntity, null, true,
              withTrans: true);
          if (!newObject.success) return newObject;
        }
      }

      // delete the removed items

      var removedIds = oldIds.where((s) => !ids.contains(s));
      if (removedIds != null && removedIds.length > 0) {
        //var queryToRemove = EntityService.getEntity(middleEntityType, parentId, "E", prop.middlePropertyName);
        var filter = EntityFilter(
            value: parentId,
            operatorId: "E",
            propertyName: prop.middlePropertyName);
        for (var i in removedIds) {
          var filter2 = EntityFilter(
              value: i,
              operatorId: "E",
              propertyName: prop.middleOtherPropertyName);
          await DatabaseService.removeRange(prop.entityName, [filter, filter2],
              withTrans: withTrans);
        }
      }

      // update the changes
      if (prop.isWithData) {
        var needUpdateItems = oldItems.data
            .where((s) => ids.contains(s[prop.middleOtherPropertyName]));

        for (var needUpdateItem in needUpdateItems) {
          dynamic oldItem;
          var id = needUpdateItem[prop.middleOtherPropertyName];

          oldItem = newItems.firstWhere((s) => s[newItemIdProp] == id);

          for (var p in middlePropertyStructure.properties) {
            if (p.propertyType == PropertyTypes.ManyToMany) {
              var resultAddMany = await _updateManyToManyEntity(
                  p as PropertyManyToMany, oldItem, needUpdateItem['id']);
              if (!resultAddMany.success) {
                return resultAddMany;
              }
            } else {
              needUpdateItem[p.dbName] = oldItem[p.dbName];
            }
          }
        }
      }

      return OperationResult(success: true);
    } catch (err, trace) {
      print('XXXXXXXXXXXXXXXXXXX ManytoManyError');
      print(err);
      print(trace);
      return new OperationResult(success: false);
    }
  }
}
