part of tecfy_core_client_package;

class InitService {
  static List<MenuItem> menuItems;

  static Future<dynamic> initTest() async {
    print('start init');
    TecfyCoreApp.packageInfo =
        PackageInfo(appName: '', version: '0.0.0', buildNumber: '0');
    await ConfigService.init();
    print('start db');
    // await DatabaseService.init();
    print('start logs');
    await LogService.init();
    print('start account');
  }

  static Future<dynamic> initTestAfter(context) async {
    print('start init');
    TecfyCoreApp.packageInfo =
        PackageInfo(appName: '', version: '0.0.0', buildNumber: '0');
    await ConfigService.init();
    print('start db');

    print('start logs');
    //await LogService.init();
    print('start account');
    await AccountService.init(context);
    print('finish account');
    return null;
  }

  static Future<dynamic> init(context) async {
    SoundService.init();
    try {
      TecfyCoreApp.packageInfo =
          await PackageInfo.fromPlatform().timeout(Duration(seconds: 1));
    } catch (err) {
      TecfyCoreApp.packageInfo =
          PackageInfo(appName: '', version: '0.0.0', buildNumber: '0');
    }
    await ConfigService.init();
    print('start db');
    await DatabaseService.init();
    print('start logs');
    await LogService.init();
    print('start account');
    await AccountService.init(context);
    print('finish account');
    return null;
  }

  static Future afterLoginInit() async {
    try {
      // TODO: move the following Maps lines to another place
      //GoogleMap.init('AIzaSyDWpCWWmDWlDneu2Mpc-OppySxvbDeK80c');
      WidgetsFlutterBinding.ensureInitialized();
      await initMenu();
      DatabaseService.init();
      NotificationAppService.init();
      MessageService.init();
      WebSocketService.init();
      MessageService.getNewMessages();
    } catch (err) {
      throw "XXXX afterLoginInit() Error: $err";
    }
  }

  static initMenu() async {
    try {
      var result = await TecfyBasicApp.apiService
          .httpGetDynamic('entity/GetEntityCategories');
      if (!result.success) {
        result.data =
            jsonDecode(ConfigService.getValueString('EntityCategory'));
        result.success = (result.data != null);
      }

      if (result.success) {
        ConfigService.setValueString('EntityCategory', jsonEncode(result.data));
        EntityService.categories = [];
        for (var cat in result.data) {
          EntityService.categories.add(EntityCategory.fromMap(cat));
        }
        await EntityService.initEntities();
        int ci = 100;
        int i = 100;

        print(
            'EntityService.categories length is ${EntityService.categories.length}');

        menuItems = EntityService.categories
            .where(
                (e) => e.entities.any((ee) => ee.permission?.canView ?? false))
            .map<MenuItem>((c) {
          var menu = MenuItem(
              title: c.name,
              orderId: ci++,
              subMenuItems: c.entities
                  .where((ee) => ee.permission?.canView ?? false)
                  .map((e) => MenuItem(
                      orderId: i++,
                      title: e.name,
                      onClick: (context) {
                        var isMobile = LayoutService.isMobile(context);
                        if (isMobile) Navigator.of(context).pop();
                        AppNavigator.navigateTo(
                            context, () => EntityListPage(e.dbName),
                            animationType: isMobile
                                ? AnimationTypes.slideLeft
                                : AnimationTypes.opacity);
                      }))
                  .toList());
          i = 100;
          return menu;
        }).toList();

        print('menuItems length ${menuItems.length} after appling permissions');
        // menuItems.insert(0, MenuItem(title: 'Main Page',orderId: 0,onClick: (context){
        //   AppNavigator.navigateTo(context, () => DashboardPage(),replaceAll: true);
        // }));

        menuItems.add(MenuItem(
            orderId: ci++,
            title: 'Reports',
            onClick: (context) {
              if (LayoutService.isMobile(context)) Navigator.pop(context);
              AppNavigator.navigateTo(context, () => ReportListPage());
            }));

        if (LocalConfigService.user.isSuperuser) {
          EntityService.entities['company'].entityType = EntityTypes.MenuItem;
          EntityService.entities['company'].permission =
              Permission.fullAccess();
          menuItems.add(
            MenuItem(
                orderId: ci++,
                title: 'Companies & Accounts',
                onClick: (context) {
                  if (LayoutService.isMobile(context)) Navigator.pop(context);
                  AppNavigator.navigateTo(
                      context, () => EntityListPage('company'));
                }),
          );
        }
        // menuItems.add(MenuItem(
        //     orderId: ci++,
        //     title: 'Settings',
        //     onClick: (context) {
        //       if(LayoutService.isMobile(context) )Navigator.pop(context);
        //       AppNavigator.navigateTo(context, () => SettingPage());
        //     }));
        // menuItems.add(MenuItem(
        //     orderId: ci++,
        //     title: 'Server Connections',
        //     onClick: (context) {
        //       if(LayoutService.isMobile(context) )Navigator.pop(context);
        //       AppNavigator.navigateTo(context, () => EntityListPage('_ServerConnections'));
        //     }));
        // menuItems.add(MenuItem(
        //     orderId: ci++,
        //     title: 'Logout',
        //     onClick: (context) {
        //       AccountService.signout(context);
        //     }));

        //print('categories : ${EntityService.categories.length}');
      } else {
        print('XXXXXX get entities failed : ${result.message}');
        menuItems = [
          MenuItem(
              orderId: 500,
              title: 'Settings',
              onClick: (context) {
                if (LayoutService.isMobile(context)) Navigator.pop(context);
                AppNavigator.navigateTo(context, () => SettingPage());
              }),
          MenuItem(
              orderId: 600,
              title: 'Server Connections',
              onClick: (context) {
                if (LayoutService.isMobile(context)) Navigator.pop(context);
                AppNavigator.navigateTo(
                    context, () => EntityListPage('_ServerConnections'));
              }),
          MenuItem(
              orderId: 1000,
              title: 'Logout',
              onClick: (context) {
                AccountService.signout(context);
              })
        ];
      }

      if (TecfyCoreApp.menuService != null)
        menuItems = TecfyCoreApp.menuService.initMenuItems(menuItems);

      menuItems.sort((a, b) => a.orderId - b.orderId);
    } catch (err, stake) {
      print('XXXXXXXXXX  initMenu : $err');
      print(stake);
      throw "init menu: $err";
    }
  }
}
