part of tecfy_core_client_package;

class EntityListSearchComponent extends StatefulWidget {
  final Entity entity;
  final String url;
  final void Function(dynamic) onClick;
  final dynamic urlFilterData;
  final Map<String, dynamic> data;
  final PropertyDropdown property;

  EntityListSearchComponent(
      {this.property,
      this.onClick,
      this.entity,
      this.data,
      this.url,
      this.urlFilterData})
      : assert(entity != null);
  @override
  _EntityListSearchComponentState createState() =>
      _EntityListSearchComponentState();
}

class _EntityListSearchComponentState extends State<EntityListSearchComponent> {
  RefreshController refreshController; // for pull to refrsh
  List<dynamic> data;
  List<dynamic> dataFilterd;
  Entity entity;
  int pageIndex = 0;
  bool noMoreData = false;
  @override
  void initState() {
    refreshController = RefreshController(initialRefresh: false);
    refreshController.footerMode.addListener(_scrollListner);

    //print(widget.entity);
    entity = widget.entity;
    //print(entity);
    getList().then((result) {
      if (result.success) {
        data = result.data
            .where((d) =>
                !widget.property.isFilter ||
                widget.data[widget.property.filterValueField] ==
                    d[widget.property.filterField])
            .toList();
        setState(() {});
      } else
        result.showMessage(context);
    });
    super.initState();
  }

  @override
  dispose() {
    refreshController?.footerMode?.removeListener(_scrollListner);
    refreshController?.dispose();
    super.dispose();
  }

  _scrollListner() async {
    print('==========on load ====== ${refreshController.footerStatus}');
    if (refreshController.footerStatus != LoadStatus.loading) return;

    pageIndex++;
    var result = await EntityService.getData(widget.entity,
        pageIndex: pageIndex, activeOnly: true);
    var success = result.success && result.data != null;
    if (success) {
      data.addAll(result.data);
      refreshController.loadComplete();
      noMoreData = result.data.length < EntityService.pageSize;
      refreshController.refreshCompleted(
          resetFooterState: result.data.length < EntityService.pageSize);
    } else {
      refreshController.loadFailed();
    }
    setState(() {});
  }

  Future<OperationResult<List<Map<String, dynamic>>>> getList() async {
    try {
      print(
          '+++++++++++ ${widget.property.filterField} , ${widget.property.filterValueField},  ${widget.data[widget.property.filterValueField]}');
      if (widget.url == null || widget.url.length == 0) {
        if (!widget.property.isFilter) {
          print('no filter');
          return await EntityService.getData(widget.entity, activeOnly: true);
        } else {
          if (widget.property.filterField.isNotEmpty) {
            return await EntityService.getData(entity,
                pageIndex: 0,
                activeOnly: true,
                filter: [
                  EntityFilter(
                      operatorId: 'E',
                      value: widget.data[widget.property.filterValueField],
                      property: entity.properties.firstWhere((element) =>
                          element.dbName == widget.property.filterField))
                ]);
          } else if (widget.property.filterQuery.isNotEmpty) {
            var query = widget.property.filterQuery
                .split('&')
                .map<EntityFilter>((e) {
                  var op = 'E';
                  var opO = '=';
                  if (e.contains('=')) {
                    op = 'E';
                    opO = '=';
                  }
                  if (e.contains('!=')) {
                    op = 'NE';
                    opO = '!=';
                  }
                  if (e.contains('>')) {
                    op = 'G';
                    opO = '>';
                  }
                  if (e.contains('>=')) {
                    op = 'GE';
                    opO = '>=';
                  }
                  if (e.contains('<')) {
                    op = 'L';
                    opO = '<';
                  }
                  if (e.contains('<=')) {
                    op = 'LE';
                    opO = '<=';
                  }
                  var splitValue = e.split(opO);
                  return EntityFilter(
                      operatorId: op,
                      value: EntityService.getValue(widget.data,
                          EntityService.lowerFirst(splitValue.last)),
                      property: entity.properties.firstWhere((element) =>
                          element.dbName ==
                          EntityService.lowerFirst(splitValue.first)));
                })
                .where((e) => e.value != null)
                .toList();
            return await EntityService.getData(entity, filter: query);
          } else if (widget.property.filterField.isEmpty &&
              widget.property.filterQuery.isEmpty) {
            if (widget.property.filterValueField.toString().contains('.')) {
              var parentPropName =
                  widget.property.filterValueField.split('.')[0];
              var childPropName =
                  widget.property.filterValueField.split('.')[1];

              var parentData =
                  widget.data[widget.property.lowerFirst(parentPropName)];
              print('------ Parent Data $parentPropName $parentData');
              var childData =
                  parentData[widget.property.lowerFirst(childPropName)];
              print('------ Child Data $childPropName $childData');
              var parentEntity = EntityService.getEntity(parentPropName);
              var childProperty = parentEntity.properties.firstWhere(
                  (e) => e.dbName.toLowerCase() == childPropName.toLowerCase());
              if (childProperty is PropertyOtherEntityMany) {
                if (childData == null) {
                  var result = await EntityService.getData(
                      EntityService.getEntity(childProperty.entityName),
                      activeOnly: true,
                      filter: [
                        EntityFilter(
                            operatorId: "E",
                            propertyName: parentEntity.dbName + "Id",
                            value: parentData["id"])
                      ]);
                  if (result.success &&
                      result.data != null &&
                      result.data.length > 0 &&
                      childProperty.entityName.toLowerCase() !=
                          widget.entity.dbName.toLowerCase()) {
                    print('------------ got data ${result.data.length}');
                    var childEntity =
                        EntityService.getEntity(childProperty.entityName);
                    var childRelationProperty = childEntity.properties
                        .firstWhere((e) =>
                            e.propertyType == PropertyTypes.Dropdown &&
                            (e as PropertyDropdown).entityName.toLowerCase() ==
                                widget.entity.dbName.toLowerCase());
                    print(result.data);
                    List<Map<String, dynamic>> lst = [];
                    for (var d in result.data) {
                      var dd = d[childRelationProperty.dbName.substring(
                          0, childRelationProperty.dbName.indexOf("Id"))];
                      if (dd == null) {
                        dd = await EntityService.getOne(widget.entity.dbName,
                            d[childRelationProperty.dbName]);
                      }
                      if (dd != null) lst.add(dd);
                    }
                    result.data = lst;
                  } else {
                    print('XXXXXXXXXXXXXXXXXXXXXXX not valid data');
                    print(result.success);
                    print(childProperty.entityName);
                    print(widget.entity.dbName);
                    print(result.data);
                  }
                  return result;
                } else {
                  print('--------------- child data not null');
                  var childEntity =
                      EntityService.getEntity(childProperty.entityName);
                  var childRelationProperty = childEntity.properties.firstWhere(
                      (e) =>
                          e.propertyType == PropertyTypes.Dropdown &&
                          (e as PropertyDropdown).entityName.toLowerCase() ==
                              widget.entity.dbName.toLowerCase());
                  // childData = childData
                  //     .map((e) => e[childRelationProperty.dbName.substring(
                  //         0, childRelationProperty.dbName.indexOf("Id"))])
                  //     .toList();

                  List<Map> lst = [];
                  for (var d in childData) {
                    var dd = d[childRelationProperty.dbName.substring(
                        0, childRelationProperty.dbName.indexOf("Id"))];
                    if (dd == null) {
                      dd = await EntityService.getOne(widget.entity.dbName,
                          d[childRelationProperty.dbName]);
                    }
                    if (dd != null) lst.add(dd);
                  }
                  childData = lst;
                  return OperationResult(success: true, data: childData);
                }
              } else {
                print('------------------prop not other many to many!');
              }
            } else {
              print('---------------filter field doesn noit has (.)');
              // TODO handel this case
              return OperationResult(success: false);
            }
          }
        }
      } else {
        print('================ Filter dropdown from URL =========');
        if (widget.urlFilterData == null)
          return OperationResult(success: false);
        var url = widget.url;
        for (var p in widget.urlFilterData.keys) {
          print('p=$p     value= ${widget.urlFilterData[p].toString()}');
          url = url.replaceAll('{$p}', widget.urlFilterData[p].toString());
        }
        return await TecfyBasicApp.apiService.httpGetDynamic(url);
      }
    } catch (err, trace) {
      print('XXXXXXXXXXXXX entity search list -> getList XXXXXXXXXX');
      print(err);
      print(trace);
      return OperationResult(success: false, message: err.toString());
    }
  }

  Future<void> onRefresh() async {
    pageIndex = 0;
    var result = await getList();
    if (result.success) {
      data = result.data
          .where((d) =>
              !widget.property.isFilter ||
              widget.data[widget.property.filterValueField] ==
                  d[widget.property.filterField])
          .toList();
      setState(() {});
      noMoreData = result.data.length < EntityService.pageSize;
      refreshController.refreshCompleted(
          resetFooterState: result.data.length < EntityService.pageSize);
    } else {
      refreshController.refreshFailed();
      result.showMessage(context);
    }
  }

  onLocalSearch(String searchText) async {
    try {
      if (searchText.isEmpty)
        dataFilterd = null;
      else if (ConnectionService.isOfflineMode) {
        List<EntityFilter> f = [];
        for (var prop in entity.properties.where((p) => p.isSearchable)) {
          f.add(EntityFilter(
              propertyName: prop.dbName,
              operatorId: prop.valueType == ValueTypes.Number ? 'E' : 'C',
              value: searchText));
        }
        var d = (await EntityService.getData(entity,
                filter: f, pageIndex: pageIndex, filterOperator: 'or'))
            .data;
        if (d != null)
          dataFilterd = d;
        else
          dataFilterd = [];
      } else {
        dataFilterd = data
            .where((d) => entity.gridProperties
                .any((p) => d[p.dbName].toString().contains(searchText)))
            .toList();
      }
      setState(() {});
    } catch (err) {
      print('XXXXXXXXXXX Local Search Error: $err');
    }
  }

  onServerSearch(String searchText) async {
    try {
      if (searchText.isEmpty)
        dataFilterd = null;
      else {
        var searchProp =
            entity.gridProperties.where((e) => e.isDisplayField).first;
        var result = await EntityService.getData(entity,
            pageIndex: 0,
            activeOnly: true,
            filter: [
              EntityFilter(
                  operatorId: 'C', value: searchText, property: searchProp)
            ]);
        if (!result.success) {
          AppNavigator.showMessage(context, 'Server Error');
        } else
          dataFilterd = result.data;
      }
      setState(() {});
    } catch (err) {
      print('XXXXXXXXXXX Server Search Error: $err');
    }
  }

  @override
  Widget build(BuildContext context) {
    return SizedBox(
        width: 300,
        child: Column(
          mainAxisAlignment: MainAxisAlignment.start,
          crossAxisAlignment: CrossAxisAlignment.stretch,
          children: <Widget>[
            SearchBox(
                onChange: (v) => onLocalSearch(v),
                onSubmit: (v) => onServerSearch(v)),
            Expanded(
                child: data == null
                    ? Center(
                        child: CircularProgressIndicator(
                            valueColor: AlwaysStoppedAnimation<Color>(
                                ThemeService.mainColor)))
                    : SmartRefresher(
                        enablePullDown: true,
                        enablePullUp: true,
                        onRefresh: onRefresh,
                        controller: refreshController,
                        header: WaterDropHeader(complete: Icon(Icons.done)),
                        footer: CustomFooter(
                          builder: (BuildContext context, LoadStatus mode) {
                            Widget body;
                            if (mode == LoadStatus.idle && !noMoreData) {
                              if (LayoutService.isMobile(context)) {
                                body = Text("pull up load");
                              } else {
                                body = TextButton(
                                    onPressed: () {
                                      refreshController.requestLoading();
                                      //_scrollListner();
                                    },
                                    child: Text("click to load mode"));
                              }
                            } else if (mode == LoadStatus.loading) {
                              body = CupertinoActivityIndicator();
                            } else if (mode == LoadStatus.failed) {
                              body = Text("Load Failed!Click retry!");
                            } else if (mode == LoadStatus.canLoading) {
                              body = Text("release to load more");
                            } else {
                              body = Text("No more Data");
                            }
                            return Container(
                              height: 55.0,
                              child: Center(child: body),
                            );
                          },
                        ),
                        child: ListView(
                            children: (dataFilterd ?? data ?? [])
                                .map<Widget>((d) => EntityListTileComponent(
                                    entity, d,
                                    onClick: widget.onClick))
                                .toList())))
          ],
        ));
  }
}
