part of tecfy_core_client_package;

class ListFormProperty extends StatefulWidget {
  final String entityId;
  final String label;
  final bool isRequired;
  final bool readOnly;
  final List<dynamic> data;
  final void Function(List<dynamic>) onChanged;
  ListFormProperty(
      {this.entityId,
      this.label,
      this.isRequired,
      this.data = const [],
      this.onChanged,
      this.readOnly = false});
  @override
  _ListFormPropertyState createState() => _ListFormPropertyState();
}

class _ListFormPropertyState extends AppState<ListFormProperty> {
  Entity entity;
  ScrollController _scrollController;

  _onValueChanged(value, propName, Map<String, dynamic> data) async {
    print('inline value change! $propName');
    data[propName] = value;

    bool updated = await entity.setDefaultValues(data);

    // for (Property p in entity.editProperties) {
    //   var r = await p.isVisble(data, entity);
    //   if (r != p.isVisable) {
    //     p.isVisable = r;
    updated = true;
    //   }
    // }

    // check the calced properties
    for (PropertyNumber p in entity.calcProps) {
      if (p.calcedProps.contains(p.upperFirst(propName)) ||
          p.calcedProps.contains(propName)) {
        var vars =
            p.calcedProps.map((pro) => data[p.lowerFirst(pro)] ?? 0).toList();
        data[p.dbName] = p.getCalcResult(vars);
        setState(() {});
      }
    }

    if (updated ||
        entity.editProperties.any((p) => p.filterValueField == propName))
      setState(() {});

    widget?.onChanged(widget.data);
  }

  _onUpdateData(Map<String, dynamic> data, Property property) {
    // we need this for Ramy's application to update the records when select an Item
    if (property.propertyType == PropertyTypes.Dropdown) {
      var props = (property as PropertyDropdown).overwritePropertiesOnSelect;
      for (var key in data.keys) {
        if (props.contains(key)) _onValueChanged(data[key], key, data);
      }
      setState(() {});
    }
  }

  @override
  void initState() {
    _scrollController = ScrollController();
    entity = EntityService.getEntity(widget.entityId);
    if (entity == null)
      print(
          'XXXXXXXXXXXX  Error Cannot find entity structure  ${widget.entityId}');
    super.initState();
  }

  @override
  void dispose() {
    _scrollController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return LayoutService.bindWidget(context,
        mobileWidget: getMobile(),
        descktopWidget: getDesktop(),
        tabletWidget: getMobile());
  }

  Widget getDesktop() {
    return Column(children: [
      Scrollbar(
          isAlwaysShown: true,
          showTrackOnHover: true,
          controller: _scrollController,
          child: SingleChildScrollView(
            controller: _scrollController,
            scrollDirection: Axis.horizontal,
            child: DataTable(
              showCheckboxColumn: true,
              columns: entity.editProperties
                  .map<DataColumn>((p) => DataColumn(
                      label: Text(p.name),
                      numeric: p.valueType == ValueTypes.Number,
                      tooltip: p.nameEn))
                  .toList(),
              rows: widget.data
                  .map<DataRow>(
                    (data) => DataRow(
                      cells: entity.editProperties
                          .map<DataCell>(
                            (p) => DataCell(
                              PorpertyComponent(
                                readOnly: widget.readOnly,
                                noLabel: true,
                                property: p,
                                data: data,
                                entity: entity,
                                onChange: (v) =>
                                    _onValueChanged(v, p.dbName, data),
                                onUpdateData: (v) => _onUpdateData(v, p),
                              ),
                            ),
                          )
                          .toList(),
                    ),
                  )
                  .toList(),
            ),
          )),
      SizedBox(height: 30),
      widget.readOnly
          ? Container()
          : SizedBox(
              width: 250,
              child: AppButtonIconText(
                label: '${trans('Click to add new')} ${widget.label}',
                icon: Icon(Icons.add, color: Colors.white),
                onPressed: () async =>
                    setState(() => widget.data.add(Map<String, dynamic>())),
              ),
            )
    ]);
  }

  Widget getMobile() {
    return Column(
      children: widget.data
          .map<Widget>((item) => InLineForm(widget.entityId,
              readOnly: widget.readOnly,
              entity: entity,
              data: item,
              onChanged: (v) => widget?.onChanged(widget.data),
              onDismiss: (v) => setState(() => widget.data.remove(v))))
          .toList()
            ..add(
              widget.readOnly
                  ? Container()
                  : SizedBox(
                      width: 250,
                      child: AppButtonIconText(
                        label: '${trans('Click to add new')} ${widget.label}',
                        icon: Icon(Icons.add, color: Colors.white),
                        onPressed: () async => setState(
                            () => widget.data.add(Map<String, dynamic>())),
                      ),
                    ),
            ),
    );
  }
}
