part of tecfy_core_client_package;

class DashboardPage extends StatefulWidget {
  @override
  _DashboardPageState createState() => _DashboardPageState();
}

class _DashboardPageState
    extends AppStateView<DashboardPage, DashboardController> {
  @override
  void initState() {
    super.initState();
    _controller = DashboardController(this);
  }

  onItemTab(type) async {
    if (type.onClick == null || type.onClick.isEmpty) return;
    if (type.onClick.startsWith('EntityList:')) {
      AppNavigator.navigateTo(context,
          () => EntityListPage(type.onClick.replaceAll('EntityList:', '')));
      return;
    }
    if (type.onClick.startsWith('Report:')) {
      var result =
          await TecfyBasicApp.apiService.httpGetDynamic('Report/GetList');
      var reportName = type.onClick.replaceAll('Report:', '');
      AppNavigator.navigateTo(
          context,
          () => ReportDetailPage(
              result.data.firstWhere((s) => s['dbName'] == reportName), []));
      return;
    }
  }

  @override
  Widget build(BuildContext context) {
    return BasePage(context, 'Main Page',
        isHome: true,
        child: _controller.dashboards == null
            ? Center(
                child: CircularProgressIndicator(
                    valueColor:
                        AlwaysStoppedAnimation<Color>(ThemeService.mainColor)))
            : _controller.dashboards.length == 0
                ? Container()
                : _controller.dashboards.length == 1
                    ? getWidgetData()
                    : Column(
                        children: [
                          getFilterWidget(),
                          Expanded(child: getWidgetData())
                        ],
                      ));
  }

  Widget getFilterWidget() {
    return Row(
      children: [
        SizedBox(width: 10),
        Container(
            constraints: BoxConstraints(maxWidth: 200),
            child: DropdownButtonHideUnderline(
              child: DropdownButtonFormField(
                //decoration: InputDecoration(labelText: 'Dashboard'),
                value: _controller.activeDashboard?.id,
                onChanged: (v) {
                  _controller.activeDashboard =
                      _controller.dashboards.firstWhere((e) => e.id == v);
                  setState(() {});
                },
                items: _controller.dashboards
                    .map<DropdownMenuItem>((d) => DropdownMenuItem(
                          child: Text(d.name),
                          value: d.id,
                        ))
                    .toList(),
              ),
            ))
      ],
    );
  }

  Widget getWidgetData() {
    return Container(
      constraints: BoxConstraints(maxWidth: ThemeService.containerWidth),
      padding: EdgeInsets.only(top: 20),
      child: AnimationLimiter(
        child: SmartRefresher(
          enablePullDown: true,
          enablePullUp: false,
          controller: _controller.refreshController,
          header: WaterDropHeader(complete: Icon(Icons.done)),
          onRefresh: _controller.refreshData,
          child: ListView(
            children: [
              getWrap(_controller.activeDashboard.items
                  .where((e) => e.type == "Statistic")
                  .toList()),
              getWrap(_controller.activeDashboard.items
                  .where((e) => e.type == "Chart")
                  .toList()),
            ],
          ),
        ),
      ),
    );
  }

  getWrap(List<DashboardItemBase> items) {
    return Wrap(
      alignment: WrapAlignment.center,
      runAlignment: WrapAlignment.center,
      crossAxisAlignment: WrapCrossAlignment.start,
      children: List.generate(
        items.length,
        (index) => AnimationConfiguration.staggeredList(
          position: index,
          duration: const Duration(milliseconds: 375),
          child: SlideAnimation(
            verticalOffset: 100.0,
            horizontalOffset: 100,
            child: FadeInAnimation(
                child: items[index].type == "Statistic"
                    ? DashboardStatisticsComponenet(items[index])
                    : items[index].type == "Chart"
                        ? DashboardChartComponenet(items[index])
                        : Text(_controller
                            .activeDashboard.items[index].displayName)),
          ),
        ),
      ),
    );
  }
}
